// cclevin class header file
//
// clever 1D integration routine based on ADPINT by WJS in FORTRAN
//                              
// The routine and associated subroutines work by splitting up the 
// intervals which seem to contribute the largest error into     
// smaller intervals for integration, repeatedly.
//
// v1.0 by Martin Kimber 20 July 2000
// Comments to: martin.kimber@physics.org
//

#ifndef _C_CLEVIN_H_INCLUDED_
#define _C_CLEVIN_H_INCLUDED_
#define MAX_INT 1000 // maximum storage size for interval arrays
                     // (maxint cannot exceed MAX_INT but it can be less)

class c_clevin {
 private:
  int maxint,resultcode,numint,calculated;
  double aerr,rerr,total,errest;
  double (*fct)(double x); // function to integrate (must be global)
  // the slave functions
  void warn();
  void calc_int_and_err(double *u,double *v,double *fu,double *fv,
                        double *fw,double *err,double *result,int thi);
  int split_and_calc(double *u,double *v,double *fu,double *fv,
                     double *fw,double *err,double *result,int thi);                      
 public:
  void set_maxint(int num);
  void set_integrand(double (*ptr_func)(double x));
  void set_aerr(double absolute_error_reqd);
  void set_rerr(double relative_error_reqd);
  double do_integral(double lower,double upper);
  double get_errest();
  int check();
  c_clevin(); // constructor (initialises automatically)
};

#endif
