// This program checks that integrating the uPDF over k_t
// gives back the PDF (only true if a LO PDF is used).
// Comments to: watt(at)hep.ucl.ac.uk

#include "parton.h"
#define RELERR 0.001 // relative error allowed by cclevin

// FUNCTION PROTOTYPES:
double unint(double);

// GLOBAL VARIABLES:
class cUnintParton updf;
const sPdfConsts consts = updf.getConsts();
const sPdfContent* cont = updf.getContent();
const double* table = updf.getTable();
// global integrating object
class c_clevin integ;

int flavour;
double x,musq;

int main(int argc, char *argv[])
{
  double xmin = 1.e-4, xmax = 0.1;
  double lower = 1.e-10, upper;
  double intPDF;
  ofstream out_file;
  char filename[]="checknorm.dat";

  // Get flavour and musq from command line:
  if (argc!=3) {
    cout << "Wrong number of arguments:" << endl;
    cout << " checknorm flavour musq" << endl;
    cout << " (flavour = 0 -> u" << endl;
    cout << "            1 -> d" << endl;
    cout << "            2 -> s" << endl;
    cout << "            3 -> c" << endl;
    cout << "            4 -> b" << endl;
    cout << "            5 -> ubar" << endl;
    cout << "            6 -> dbar" << endl;
    cout << "            7 -> g)" << endl;
    return(0);
  }
  flavour = atoi(argv[1]);
  musq = atof(argv[2]);
  upper = musq;

  integ.set_rerr(RELERR); // aerr=0 by default
  integ.set_integrand(unint);

  // Write x distribution to file:
  out_file.open(filename);  
  if (out_file.bad()) {
    cerr << "Error opening " << filename << "\n";
    exit (-1);
  }

  out_file << "# x int int(unint)" << endl;

  for(int i=0;i<=100;i++) {
    // Assign value between xmin and xmax, distributed logarithmically:
    x = pow(10,log10(xmin)+(log10(xmax)-log10(xmin))*double(i)/100);

    updf.update(x,musq); intPDF = table[flavour];

    out_file  << x << " " << intPDF << " " 
	      << integ.do_integral(lower,upper) << endl;
  }

  out_file.close();

  return(0);
}

double unint(double ktsq)
{
  return updf.calc(flavour,x,ktsq,musq)/ktsq;
}
