// cParton and cUnintParton class declarations
// Comments to: watt(at)hep.ucl.ac.uk
//
// cParton is based on original code by Jeppe Andersen.
// cUnintParton is based on original code by Martin Kimber.


#ifndef _PARTON_H_INCLUDED_
#define _PARTON_H_INCLUDED_

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <math.h>
#include <fstream.h>
#include <cclevin.h>

const size_t NSUDGRIDKT=1000, NSUDGRIDMU=1000;
enum ePdfSet { MRST2001LO, CTEQ6L1, CTEQ5L, CTEQ5LPAR, GRV94LO, MRST2002NLO };
struct sPdfContent { double u,d,s,c,b,ubar,dbar,g; };
struct sPdfConsts {
  ePdfSet pdfset; size_t pdfmode,loop;
  double mcharm,mbottom,xmin,xmax,musqmin,musqmax,LQCD,aSMZ2;
};


// cParton is the class for the conventional (integrated) PDFs.
class cParton
{
 public:
  cParton(void);                             // constructor
  const sPdfConsts getConsts(void) const;    // returns consts
  const sPdfContent* getContent(void) const; // returns pointer to content
  const double* getTable(void) const;        // returns table
  void update(double x, double musq); // update parton content
  double alpha_S(double musq) const;  // alpha_S as fn. of renormalisn. scale
 protected:
  sPdfConsts consts;    // Constants specific to a particular PDF set
  sPdfContent content;  // Integrated parton distribution functions (*x)
  double* table[8];     // Pointer to content in table form
};


// cUnintParton is the class for the (doubly-)unintegrated PDFs.
// It inherits the cParton class.
class cUnintParton : public cParton
{
 public:
  cUnintParton(bool interpolate=false);  // constructor
  ~cUnintParton(void); // destructor
  // Calculate the unintegrated PDF:
  double calc(size_t flavour,double x,double ktsq,double musq);
  // Calculate the doubly-unintegrated PDF:
  double calc(size_t flavour,double x,double z,double ktsq,double musq);
  double quarkSudakov(double ktsq,double musq);
  double gluonSudakov(double ktsq,double musq);
 private:
  class c_clevin integrate1D;
  bool interpolate; // Interpolate Sudakovs from (ktsq,musq) grid or not
  double** qSudGrid;// [NSUDGRIDKT][NSUDGRIDMU];
  double** gSudGrid;// [NSUDGRIDKT][NSUDGRIDMU];
};


// Error and warning global functions:

template <typename Type>
inline void error(const char* function, const char* variable, Type value)
{ cerr << endl << "Error in " << function << ": invalid " << variable 
       << " = " << value << endl; exit(0); }

template <typename Type>
inline void warning(const char* function, const char* variable, Type value)
{ cerr << endl << "Warning in " << function << ": invalid " << variable 
       << " = " << value << endl; }

#endif
