// This program demonstrates the usage of the cUnintParton class.
// Comments to: watt(at)hep.ucl.ac.uk

#include "parton.h"

int main (void)
{
  class cUnintParton updf;
  const sPdfContent* content = updf.getContent();
  const sPdfConsts consts = updf.getConsts();
  const double* table = updf.getTable();
  const double MZsq = pow(91.1876,2);
  double x,z,ktsq,musq;

  x = 0.01; z=0.1; ktsq = 10.; musq = 100.;
  updf.update(x,musq);

  // First, demonstrate the interface to the conventional integrated PDFs.
  cout << "Integrated PDFs at x = " << x << ", musq = " << musq << ":" << endl;
  cout << "xu(x.musq) = " << content->u << " or " << table[0] << endl;
  cout << "xd(x,musq) = " << content->d << " or " << table[1] << endl;
  cout << "xs(x.musq) = " << content->s << " or " << table[2] << endl;
  cout << "xc(x,musq) = " << content->c << " or " << table[3] << endl;
  cout << "xb(x,musq) = " << content->b << " or " << table[4] << endl;
  cout << "xubar(x,musq) = " << content->ubar << " or " << table[5] << endl;
  cout << "xdbar(x,musq) = " << content->dbar << " or " << table[6] << endl;
  cout << "xg(x,musq) = " << content->g << " or " << table[7] << endl;
  cout << endl;

  // Now demonstrate how to calculate the uPDFs.
  cout << "Unintegrated PDFs at x = " << x << ", ktsq = " << ktsq << ", musq = " << musq << ":" << endl;
  cout << "f_u(x,ktsq,musq) = " << updf.calc(0,x,ktsq,musq) << endl;
  cout << "f_d(x,ktsq,musq) = " << updf.calc(1,x,ktsq,musq) << endl;
  cout << "f_s(x,ktsq,musq) = " << updf.calc(2,x,ktsq,musq) << endl;
  cout << "f_c(x,ktsq,musq) = " << updf.calc(3,x,ktsq,musq) << endl;
  cout << "f_b(x,ktsq,musq) = " << updf.calc(4,x,ktsq,musq) << endl;
  cout << "f_ubar(x,ktsq,musq) = " << updf.calc(5,x,ktsq,musq) << endl;
  cout << "f_dbar(x,ktsq,musq) = " << updf.calc(6,x,ktsq,musq) << endl;
  cout << "f_g(x,ktsq,musq) = " << updf.calc(7,x,ktsq,musq) << endl;
  cout << endl;
  
  // Now demonstrate how to calculate the duPDFs.
  cout << "Doubly-unintegrated PDFs at x = " << x << ", z = " << z << ", ktsq = " << ktsq << ", musq = " << musq << ":" << endl;
  cout << "f_u(x,z,ktsq,musq) = " << updf.calc(0,x,z,ktsq,musq) << endl;
  cout << "f_d(x,z,ktsq,musq) = " << updf.calc(1,x,z,ktsq,musq) << endl;
  cout << "f_s(x,z,ktsq,musq) = " << updf.calc(2,x,z,ktsq,musq) << endl;
  cout << "f_c(x,z,ktsq,musq) = " << updf.calc(3,x,z,ktsq,musq) << endl;
  cout << "f_b(x,z,ktsq,musq) = " << updf.calc(4,x,z,ktsq,musq) << endl;
  cout << "f_ubar(x,z,ktsq,musq) = " << updf.calc(5,x,z,ktsq,musq) << endl;
  cout << "f_dbar(x,z,ktsq,musq) = " << updf.calc(6,x,z,ktsq,musq) << endl;
  cout << "f_g(x,z,ktsq,musq) = " << updf.calc(7,x,z,ktsq,musq) << endl;
  cout << endl;
  
  // Finally, demonstrate how to access the Sudakov form factors.
  cout << "Sudakov form factors at ktsq = " << ktsq << ", musq = " << musq << ":" << endl;
  cout << "T_q(ktsq,musq) = " << updf.quarkSudakov(ktsq,musq) << endl;
  cout << "T_g(ktsq,musq) = " << updf.gluonSudakov(ktsq,musq) << endl;
  cout << endl;

  //  Get alpha_S(MZ^2) for this parton set:
  cout << "alpha_S(MZ^2) = " << updf.alpha_S(MZsq) << endl;

  // Plot alpha_S as a function of the renormalisation scale.
  // (Check it's continuous at flavour thresholds.)
  ofstream out_file;
  char filename[]="alphas.dat";
  double alphaS;
  const size_t nkt=100;
  const double ktsqmin=1., ktsqmax=100.;
  out_file.open(filename);
  if (out_file.bad()) error("testpdf","problem opening",filename);
  out_file << "# kt^2 alpha_S(kt^2)" << endl;
  for(size_t i=0;i<nkt;i++) {
    ktsq=pow(10,log10(ktsqmin) + i*(log10(ktsqmax)-log10(ktsqmin))/(nkt-1));
    alphaS=updf.alpha_S(ktsq);
    out_file << ktsq <<" "<< alphaS << endl;
  }
  out_file.close();

  return (0);
}
